@@ -1,6 +1,11 @@
 #!/bin/bash
 set -eux
 
+if [ "$(uname -s)" != "Linux" ]; then
+    echo "Please use the GitHub Action."
+    exit 1
+fi
+
 SCRIPT_DIR="$( dirname "$0" )"
 cd $SCRIPT_DIR/..
 
